//---------------------------------------------------------------------------
//  watek odczytu danych z USB

#include "ThreadComm.h"
#include "usb.h"
#include <stdio.h>
#include "..\miniscope\device\MiniscopeDevice.h"
#include "ScopedLock.h"
#include "armscope_cmd.h"

#pragma package(smart_init)

bool writeLock;

__fastcall ThreadComm::ThreadComm(bool CreateSuspended)
		: callbackLog(NULL), callbackData(NULL), callbackTrigger(NULL),
		iBufferSamples(512),
		connected(true)
{
}
//---------------------------------------------------------------------------


void ThreadComm::log(char *lpData, ...)
{
    static char buf[256]; //determines max message length
	va_list ap;
	va_start(ap, lpData);
	vsnprintf(buf, sizeof(buf), lpData, ap);
	va_end(ap);
	buf[sizeof(buf)-1] = 0;

	if (callbackLog)
		callbackLog(buf);
}

//---------------------------------------------------------------------------
DWORD WINAPI ThreadProc(LPVOID data)
{
    ThreadComm *thrRead = (ThreadComm*)data;
	int received = 0;
	int recv_timeout = 30;

	while (thrRead->connected)
	{
		enum { E_USB_RCV_SIZE = 14336 };
		static unsigned char usb_rcv[E_USB_RCV_SIZE];
		int ret = usb_bulk_read( usb_handle, 0x82, usb_rcv, thrRead->GetBufferSize(), 20 );
		if (ret < 0) {
			if (ret != -116)	//timeout
			{
				thrRead->log("usb_bulk_read: status = %d, %s", ret, usb_strerror());
				thrRead->connected = false;
				if (thrRead->callbackConnect)
					thrRead->callbackConnect(DEVICE_DISCONNECTED, "usb_bulk_read failed, disconnected");
			}
		}

		if (ret >= 7)
		{
			if (usb_rcv[0]==0xFF && usb_rcv[1]==0 && usb_rcv[2]==0xFF && usb_rcv[3]==0 && usb_rcv[4]==255 &&
					usb_rcv[5]==0 && usb_rcv[6]==255)
			{
			}
			else
			{
				if (received == 0)
				{
					if (thrRead->callbackTrigger)
						thrRead->callbackTrigger();
				}
				static short sendBuffer[E_USB_RCV_SIZE];
				for (int i=0; i<ret; i++)
					sendBuffer[i] = 127 - usb_rcv[i];
				received += ret;
				if (thrRead->callbackData)
					thrRead->callbackData(sendBuffer, ret * sizeof(short));
				if (received >= thrRead->GetBufferSize())
				{
					received = 0;
					thrRead->EndOfFrame();
				}
				recv_timeout = 30;
			}
		} else {
			if (received)
			{
				recv_timeout--;
				if (!recv_timeout)
				{
					received = 0;
					thrRead->EndOfFrame();
					OutputDebugString("Looks like a frame receiving timeout");
				}
			}
		}

		{
			ScopedLock<Mutex> lock(thrRead->mutexSend);
			if (!thrRead->que_cmd.empty()) {
				ThreadComm::Cmd scmd = thrRead->que_cmd.front();
				thrRead->que_cmd.pop_front();
				char buffer[3];
				buffer[0] = scmd.cmd;
				buffer[1] = scmd.val;
				buffer[2] = 0x00;
				ret = usb_bulk_write(usb_handle, 0x01, buffer, 3, 500);
				if (ret < 3)
					OutputDebugString("Write error");
				Sleep(50);
			} else {
				if (received == 0) //frame transmission not started yet
				{
					char buffer[3];
					buffer[0] = CMD_FUNCTION;
					buffer[1] = CMD_FUNCTION_FETCH_DATA;
					buffer[2] = 0x00;
					ret = usb_bulk_write(usb_handle, 0x01, buffer, 3, 500);
					if (ret < 3)
						OutputDebugString("Write error");
					Sleep(50);
				}
			}
		}
	}
	return 0;
}
//---------------------------------------------------------------------------

void ThreadComm::Execute(void)
{
    DWORD dwtid;
    Thread = CreateThread(NULL, 0, ThreadProc, this, 0, &dwtid);
}

void ThreadComm::Terminate(void)
{
    /** \todo Avoid TerminateThread as dangerous */
	TerminateThread(Thread, 0);
}

int ThreadComm::SendCmd(unsigned char cmd, unsigned char val, bool bImmediate)
{
	if (!connected)
		return -1;
	if (!bImmediate)
	{
		ScopedLock<Mutex> lock(mutexSend);
		Cmd scmd;
		scmd.cmd = cmd;
		scmd.val = val;
		que_cmd.push_back(scmd);
	}
	else
	{
		char buffer[3];
		buffer[0] = cmd;
		buffer[1] = val;
		buffer[2] = 0x00;
		if (usb_handle == NULL)
			return -2;
		ret = usb_bulk_write(usb_handle, 0x01, buffer, 3, 2000);
		if (ret < 3)
			return -3;
	}
	return 0;
}

void ThreadComm::EndOfFrame(void)
{
	if (bContinuousTrigger)
		SendCmd(CMD_FUNCTION, CMD_FUNCTION_TRIGGER_EXECUTE);
}

